/*
 *   This file is part of mpck, a program to check MP3 files for errors
 *   
 *   Copyright (C)  2003  Sjoerd Langkemper
 *   
 *   mpck is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************
 *
 *   bufio.c - buffered cfopen, cfread, cfclose, cftell and cfseek 
 * 
 */

#include <stdio.h>
#include "mpck.h" 
#include "bufio.h"
#include <string.h>

#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif

#define USE_BUFFERED_IO
#ifdef USE_BUFFERED_IO

#define reminbuf(c) (c->buffer+c->buflen-c->bufpnt)

/* init the cfile struct so that the buffer seems empty */
void init_cfile(CFILE * c) {
	c->bufpnt=c->buffer+CBUFSIZE;
	c->buflen=CBUFSIZE;
	c->inmem=-CBUFSIZE;
}

/*
int reminbuf(CFILE * c) {
	return c->buffer+c->buflen-c->bufpnt;
}
*/

int cfload(CFILE * c, long offset) {
	int read;
	fseek(c->fp, offset, SEEK_SET);
	read=fread(c->buffer, 1, CBUFSIZE, c->fp);
	c->buflen=read;
	c->inmem=offset;
	c->bufpnt=c->buffer;
	return read;
}
	

CFILE * cfopen(const char * filename, char * mode) {
	CFILE *c=(CFILE *)malloc(sizeof(CFILE)+CBUFSIZE);
	if (c==NULL) return NULL;

	c->buffer=((char *)c+sizeof(CFILE));

	c->fp=fopen(filename, "rb");
	if (c->fp==NULL) {
		free(c);
		return NULL;
	}

	init_cfile(c);
	return c;
}

size_t cfread(char * outbuf, size_t size, CFILE * c) {
	int res;
	int readnow;
	int toread=size;
	
	do {
		if (reminbuf(c)<=0) {
			// load new data into the buffer
			res=cfload(c, c->inmem+CBUFSIZE);
			if (!res) break;
		}
		readnow=MIN(reminbuf(c), toread);
		memcpy(outbuf+size-toread, c->bufpnt, readnow);
		toread-=readnow;
		c->bufpnt+=readnow;
	} while (toread);

	return size-toread;
}

int cfsetpos(CFILE * c, long pos) {
	int res;
	if ((pos>=c->inmem)&&(pos<c->inmem+c->buflen)) {
		c->bufpnt=c->buffer+(pos-c->inmem);
		return 0;
	} else {
		res=cfload(c, pos);
		if (res) return 0;
		    else return -1;
	}
}

int cfseek(CFILE * c, long offset, int whence) {
	long curpos;
	long absoffset;
	if (whence==SEEK_SET) absoffset=offset; 
	if (whence==SEEK_CUR) {
		curpos=cftell(c);
		absoffset=curpos+offset;
	}
	if (whence==SEEK_END) {
		fseek(c->fp, offset, SEEK_END);
		absoffset=ftell(c->fp);
	}
	if (absoffset>=0) return cfsetpos(c, absoffset);
		     else return -1;
}

long cftell(CFILE * c) {
	return c->inmem+c->bufpnt-c->buffer;
}

int cfeof(CFILE * c) {
	return ((reminbuf(c)==0)&&(feof(c->fp)));
}

int cfclose(CFILE * c) {
	int res;
	res=fclose(c->fp);
	free(c);
	return res;
}


#else /* not USE_BUFFERED_IO */

CFILE * cfopen(const char * filename, char * mode) {
	CFILE *c=(CFILE *)malloc(sizeof(CFILE));
	if (c==NULL) return NULL;
	c->fp=fopen(filename, "rb");
	if (c->fp==NULL) {
		free(c);
		return NULL;
	}
	return c;
}

size_t cfread(char * outbuf, size_t size, CFILE * c) {
	return fread(outbuf, 1, size, c->fp);
}

int cfseek(CFILE * c, long offset, int whence) {
	return fseek(c->fp, offset, whence);
}

long cftell(CFILE * c) {
	return ftell(c->fp);
}

int cfeof(CFILE * c) {
	return feof(c->fp);
}

int cfclose(CFILE * c) {
	return fclose(c->fp);
}

#endif /* USE_BUFFERED_IO */
