/*
 *   This file is part of mpck, a program to check MP3 files for errors
 *   
 *   Copyright (C)  2003  Sjoerd Langkemper
 *   
 *   mpck is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************
 *
 *   main.c - this is where the program starts
 * 
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include "mpck.h"
#include <sys/stat.h>
#include <sys/types.h>
#include <string.h>

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_DIRENT_H
#include <dirent.h>
#endif

#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_MEMORY_H
#include <memory.h>
#endif

/* some sys/stat.h do not define these macro's */
#ifndef S_ISDIR
#define S_ISDIR(mode) (((mode) & S_IFDIR)==(S_IFDIR))
#endif /* S_ISDIR */

#ifndef S_ISREG
#define S_ISREG(mode) (((mode) & S_IFREG)==(S_IFREG))
#endif /* S_ISREG */

#ifdef HAVE_WINDOWS_H
#include <windows.h>
#endif

/* Flag variables */
static int first_n_bytes=0; 
static int first_n_frames=0; 
static int recursive=0;		/* scan directories recursively		-R */
static int quiet=0;		/* only output if a MP3 is good or not	-q */
static char extention[MAXEXT];	/* only scan files with this extention	-e */

static int lasterror=0;		/* contains the value of the last error    */

/* needed for getopt */
char *
_getprogname() {
	return PROGNAME;
}

/*
 * puts stats from file (bitrate, file count) into the total stats
 */
int
parsetotalstats(total, file)
	total_info * total;
	file_info  * file;
{
	/* don't do stats for files which aren't MP3s */
	if (!file->ismp3file) return TRUE;

	total->filecount++;

	if (!file->errors) total->goodcount++;
	total->time+=file->time;
	total->totalbitrate+=file->bitrate;
	if ((!total->minbitrate)||(file->bitrate < total->minbitrate)) {
		total->minbitrate=file->bitrate;
	}
	if (file->bitrate > total->maxbitrate) {
		total->maxbitrate=file->bitrate;
	}
	return TRUE;
}

#ifdef _WIN32

/* wildcard_parse_args calls wildcard_parse_file for each argument.
 * only if _WIN32 is defined
 */
int
wildcard_parse_args(total, argv)
	total_info 	* total;
	char		**argv;
{
	int res=TRUE;

	do {
		res=res && wildcard_check_arg(total, *argv);
		argv++;
	} while (*argv);

	return res;
}



/* wildcard_parse_file 
 * only if _WIN32 is defined
 */
int
wildcard_check_arg(total, wildcardpath)
	total_info * total;
	char * wildcardpath;
{
	WIN32_FIND_DATA FindFileData;	/* struct to store the found file */
	HANDLE hFind;
	int wcplen;		/* wildcard path length 		*/
	int res;		/* function result 			*/
	char * filepart;	/* part of wcp with the filename 	*/
	char * realfilename;	/* expanded filename 			*/

	wcplen=strlen(wildcardpath);
	realfilename=(char*)malloc(wcplen+MAXPATH);
	if (realfilename==NULL) {
		error(" malloc failed while doing wildcard expansion\n");
		lasterror=ENOMEM;
		return FALSE;
	}

	strcpy(realfilename, wildcardpath);
	filepart=strrchr(realfilename, DIRSEP);
	if (filepart==NULL) {
		filepart=realfilename; /* no backslash found */
	} else {
		filepart+=1; /* skip the backslash */
	}

	hFind=FindFirstFile(wildcardpath, &FindFileData);
	if (hFind == INVALID_HANDLE_VALUE) {
//		error(" no file found: %s", wildcardpath);
		error(" %s: I searched for a while, but I cannot find this file.", wildcardpath);
		lasterror=ENOENT;
		return FALSE;
	}

	do {
		if (FindFileData.cFileName[0]!='.') { /* skip ".*" */
			strncpy(filepart, FindFileData.cFileName, MAXPATH);
			check_arg(total, realfilename);
		}
		res=FindNextFile(hFind, &FindFileData);
		if (res==0) res=GetLastError();
	} while (res!=ERROR_NO_MORE_FILES);

	FindClose(hFind);
	free(realfilename);
	return TRUE;
}

/* only if _WIN32 is defined 
 */
int
recursivecheck(total, dirname)
	total_info * total;
	char *dirname;
{
	char * wildcarddir;
	int dirlen;

	dirlen=strlen(dirname);
	wildcarddir=(char *)malloc(dirlen+MAXPATH);
	if (wildcarddir==NULL) {
		error(" malloc failed");
		lasterror=ENOMEM;
		return FALSE;
	}
	
	strcpy(wildcarddir, dirname);
	strcat(wildcarddir, "\\*");

	wildcard_check_arg(total, wildcarddir);
	free(wildcarddir);
	return TRUE;
}

#else /* !_WIN32 */

/* recursively scans all the files in dirname */
int
recursivecheck(total, dirname)
	total_info * total;
	char *dirname;
{
	DIR * dir;
	struct dirent * ent;
	char * filename;
	int dirlen;

	dirlen=strlen(dirname);
	filename=(char *)malloc(2+dirlen+256);
	if (filename==NULL) {
		// error(" malloc failed while doing recursion\n");
		error(" malloc failed while doing infinite recursion\n");
		lasterror=ENOMEM;
		return FALSE;
	}
	strcpy(filename, dirname);
	filename[dirlen]='/';


	dir=opendir(dirname);
	if (dir==NULL) {
		error(" error opening directory %s", dirname);
		lasterror=ENOENT;
		return FALSE;
	}
	while ((ent=readdir(dir))!=NULL) {
		if (ent->d_name[0]=='.') continue; /* hidden files, . and .. */
		strncpy(filename+dirlen+1, ent->d_name, 255);
		check_arg(total, filename);
	}
	closedir(dir);
	free(filename);
	
	return TRUE;
}

#endif /* !_WIN32 */

/* check_arg decides whether the file must be checked, opens it and passes
 * it to checkfile
 */
int
check_arg(total, filename)
	total_info * total;
	char * filename;
{
	struct stat _ss;
	struct stat * ss=&_ss; 

	int statres;

	file_info _file;
	file_info * file=&_file;
	init_file_info(file);

	/* stat the file */ 
	statres=stat(filename, ss);
	if (statres!=0) {
		error("stat on %s failed", filename);
		lasterror=ENOENT;
		return FALSE;	
	} 

	/* move info from ss to file */
	file->length=(int)(ss->st_size);

	/* check for file type */
	if (S_ISDIR(ss->st_mode)) {
		if (recursive) {
			recursivecheck(total, filename);
		} else {
			error(" %s is a directory; use -R for recursive checking", filename);
			lasterror=EISDIR;
		}
		return FALSE;
	} else if (!S_ISREG(ss->st_mode)) {
		/* actually a warning */
		error(" %s is not an ordinary file", filename);
	}
	
	/* check if the extention of the file matches the extention */
	if (*extention) {
		if (!extention_match(filename, extention)) {
			return FALSE;
		}
	}

	/* open file */
	if ((file->fp = cfopen(filename, "rb")) != NULL) {
		file->filename=filename;
	} else {
		error("%s: %s", filename, strerror(errno));
		lasterror=errno;
		return FALSE;
	}

	checkfile(file);

	cfclose(file->fp);

	parsetotalstats(total, file);

	return TRUE;

}

/* parse_args opens each file and calls check_arg */
int
parse_args(total, argv)
	total_info 	* total;
	char		**argv;
{
	int res=TRUE;
	do {
		res=res && check_arg(total, *argv);
		argv++;
	} while (*argv); 

	return res;
}

int
parse_options(argc, argv)
	int argc;
	char *argv[];
{	
	
	int ch;			/* option */
	int option_index;
	int verbose=0, maxname=0;

	struct option long_options[] = {
		{"version",   0, 0, 'V'},
		{"verbose",   0, 0, 'v'},
		{"bytes",     1, 0, 'b'},
		{"frames",    1, 0, 'f'},
		{"help",      0, 0, 'h'},
		{"quiet",     0, 0, 'q'},
		{"silent",    0, 0, 'q'},
		{"recursive", 0, 0, 'R'},
		{"extention", 1, 0, 'e'},
		{"maxname",   1, 0, 'm'},
		{"badonly",   0, 0, 'B'},
		{"namecheck", 0, 0, 'n'},
		{NULL, 0, 0, 0}
	};

	extention[0]=0;

	while ((ch = getopt_long(argc, argv, "Vv::b:f:hqRre:m:Bn", long_options, &option_index)) >32) {
		switch (ch) {
			case 'R':
				/* recursion */
				recursive=TRUE;
				break;
			case 'r':
				/* -r == -R -e mp3 */
				strcpy(extention, DEF_EXTENTION);
				recursive=TRUE;
				break;
			case 'V':
				/* version info */
				print_version();
				exit(0);
			case 'v':
				/*  verbose  */
				verbose++;

				if (optarg && *optarg == 'v') {
					error(" switching to mega-super-power-boosting verbosity mode...");
					verbose++;
				}
				set_verbose(verbose);
	
				break;

			case 'b':
				/*  Only scan the first # of bytes, then quit.*/
				first_n_bytes=atoi(optarg);
				if (first_n_bytes<=0) {
					error(" %s is not a valid argument to -b", optarg);
					exit(EINVAL);
				}
				set_first_n_bytes(first_n_bytes);
				break;
			case 'f':
				/* Only scan the first # of frames, then quit.*/
				first_n_frames=atoi(optarg);
				if (first_n_frames<=0) {
					error(" %s is not a valid argument to -f", optarg);
					exit(EINVAL);
				}
				set_first_n_frames(first_n_frames);
				break;
			case 'e':
				/* Only scan files with this extention */
				if (strlen(optarg)>MAXEXT) {
					error(" extention is too long");
					exit(ENAMETOOLONG);
				}
				strcpy(extention, optarg);
				break;
			case 'm':
				maxname=atoi(optarg);
				if (maxname<=0) {
					error(" %s is not a valid argument to -m", optarg);
					exit(EINVAL);
				}
				set_maxname(maxname);
				break;
			case 'h':
				print_usage();
				exit(0);
			case 'q':
				/* Only print either good or bad for a file */
				quiet=TRUE;
				set_quiet(quiet);
				break;
			case 'B':
				/* Only report bad files */
				set_badonly(TRUE);
				break;
			case 'n':
				/* Check for strange chars in filenames */
				set_namecheck(TRUE);
				break;
			case '?':
				error(" ambiguous match or an extraneous parameter");
				exit(EINVAL);
			case ':':
				error(" missing parameter");
				exit(EINVAL);
			default:
				exit(EINVAL);
		}
	}

	/* return the number of parsed words */
	return optind;
}


int
main(argc, argv)
	int argc;
	char *argv[];
{
	int opt_count;

	total_info _total;
	total_info * total=&_total;

	init_total_info(total);

	opt_count=parse_options(argc, argv);
	argv+=opt_count;	/* skip options */

	/* If there's no data after the flags, return an error. */
	if (*argv == NULL) {
//		error(" no filenames specified");
		error(" no filename supplied, your request is denied");
		lasterror=EINVAL;
		return 1;
	}

	/* -b and -f screw up the statistics */
	if (!quiet) {
		if (first_n_bytes) error(" only scanning first %d bytes of each file, which probably results in wrong statistics", first_n_bytes);
		if (first_n_frames) error(" only scanning first %d frames of each file, which probably results in wrong statistics", first_n_frames);
	}


#ifdef _WIN32

	/* in linux, wildcard expansion is done by the shell, but in windows, we
	 * have to do it ourselves.
	 */
	wildcard_parse_args(total, argv);
#else
	parse_args(total, argv);
#endif

	if ((!quiet)&&(total->filecount>1)) print_total_info(total);

	if (lasterror) {
		/* error occured */
		exit(lasterror);
	} else if (total->filecount==total->goodcount) {
		/* nothing wrong */
		exit(0);	
	} else {
		/* at least one file is damaged */
		exit(1);
	}
} /* main */ 
